from intelex import get_version, get_endpoint, get_apikey, _select_format, _count_format, _paginate_top_format, _paginate_skip_format, _generate_query_string, _filter_format, _sort_format
import os

def test_get_version():
    assert get_version() == '0.0.27'

def test_get_endpoint():
    assert get_endpoint() == os.environ['ilx_endpoint']

def test_get_aoikey():
    assert get_apikey() == os.environ['ilx_apikey']

def test_select_format():
    select_list = ['Hello', 'World', 'List']
    assert _select_format(select_list) == 'Hello, World, List'

def test_count_format_true():
    assert _count_format(True) == '$count=true'

def test_count_format_false():
    assert _count_format(False) == '$count=false'

def test_count_format_blank():
    assert _count_format() == '$count=false'

def test_paginate_top_format():
    assert _paginate_top_format(50) == '$top=50'

def test_paginate_skip_format():
    assert _paginate_skip_format(100) == '$skip=100'

def test_paginate_filter_format():
    filter_expression = 'TaskType eq \'Question\''
    assert _filter_format(filter_expression) == '$filter=TaskType eq \'Question\''

def test_sort_format():
    sort_list = ['DateCreated asc']
    assert _sort_format(sort_list) == '$orderby=DateCreated asc'


def test_generate_query_string_all():
    query_string = {
        'select': ['RecordNumber', 'Id'],
        'count': True,
        'paginate_top': 50,
        'paginate_skip': 100,
        'sort': ['DateCreated asc'],
        'filter': 'TaskType eq \'Question\''
    }
    assert _generate_query_string(query_string) == '$select=RecordNumber, Id&$count=true&$top=50&$skip=100&$orderby=DateCreated asc&$filter=TaskType eq \'Question\''


def test_generate_query_string_select():
    query_string = {
        'select': ['RecordNumber', 'Id']
    }
    assert _generate_query_string(query_string) == '$select=RecordNumber, Id'


def test_generate_query_string_sort():
    query_string = {
        'sort': ['DateCreated asc', 'DateModified']
    }
    assert _generate_query_string(query_string) == '$orderby=DateCreated asc, DateModified'


def test_generate_query_string_count():
    query_string = {
        'count': True
    }
    assert _generate_query_string(query_string) == '$count=true'


def test_generate_query_string_top():
    query_string = {
        'paginate_top': 50
    }
    assert _generate_query_string(query_string) == '$top=50'


def test_generate_query_string_skip():
    query_string = {
        'paginate_skip': 100
    }
    assert _generate_query_string(query_string) == '$skip=100'


def test_generate_query_string_filter():
    query_string = {
        'filter': 'TaskType eq \'Question\''
    }
    assert _generate_query_string(query_string) == '$filter=TaskType eq \'Question\''
    



import requests as r
import os
import json


# Private Functions

def _select_format(select_list):
    output = ', '
    output = output.join(select_list)
    return '{}'.format(output)


def _count_format(bool=False):
    if bool:
        return '$count=true'
    else:
        return '$count=false'


def _paginate_top_format(top):
    return '$top={}'.format(top)


def _paginate_skip_format(skip):
    return '$skip={}'.format(skip)


def _filter_format(filter_expression):
    return '$filter={}'.format(filter_expression)


def _sort_format(sort_list):
    output = ', '
    output = output.join(sort_list)
    return '$orderby={}'.format(output)


def _generate_query_string(query_string):
    
    result = []

    if 'select' in query_string:
        result.append('$select={}'.format(_select_format(query_string['select'])))

    if 'count' in query_string:      
        result.append('{}'.format(_count_format(query_string['count'])))

    if 'paginate_top' in query_string:
        result.append('{}'.format(_paginate_top_format(query_string['paginate_top'])))
    
    if 'paginate_skip' in query_string:
        result.append('{}'.format(_paginate_skip_format(query_string['paginate_skip'])))

    if 'sort' in query_string:
        result.append('{}'.format(_sort_format(query_string['sort'])))

    if 'filter' in query_string:
        result.append('{}'.format(_filter_format(query_string['filter'])))


    output = '&'
    output = output.join(result)
    
    return '{}'.format(output)


# Public Functions

def get_version():
    return '0.0.27'


def get_endpoint():
    ilx_endpoint = os.environ['ilx_endpoint']
    return ilx_endpoint


def get_apikey():
    ilx_apikey = os.environ['ilx_apikey']
    return ilx_apikey


def get_records(object, params=None):
    ilx_endpoint = os.environ['ilx_endpoint']
    ilx_apikey = os.environ['ilx_apikey']
    auth_header = 'Basic {}'.format(ilx_apikey)

    headers = {
        'Authorization': auth_header
    }

    if params != None:
        query = '{}/api/v2/object/{}?{}'.format(ilx_endpoint, object, _generate_query_string(params))
    else:
        query = '{}/api/v2/object/{}'.format(ilx_endpoint, object)
    
    response = r.get(query, headers=headers)
    
    return json.loads(response.text)


def get_record(object, id):
    ilx_endpoint = os.environ['ilx_endpoint']
    ilx_apikey = os.environ['ilx_apikey']
    auth_header = 'Basic {}'.format(ilx_apikey)

    headers = {
        'Authorization': auth_header
    }

    query = '{}/api/v2/object/{}({})'.format(ilx_endpoint, object, id)

    response = r.get(query, headers=headers)

    return json.loads(response.text)


def get_related_records(object, id, navigation_property):
    ilx_endpoint = os.environ['ilx_endpoint']
    ilx_apikey = os.environ['ilx_apikey']
    auth_header = 'Basic {}'.format(ilx_apikey)

    headers = {
        'Authorization': auth_header
    }

    query = '{}/api/v2/object/{}({})/{}'.format(ilx_endpoint, object, id, navigation_property)
    response = r.get(query, headers=headers)

    return json.loads(response.text)


def get_related_record(object, id, navigation_property, related_id):
    ilx_endpoint = os.environ['ilx_endpoint']
    ilx_apikey = os.environ['ilx_apikey']
    auth_header = 'Basic {}'.format(ilx_apikey)

    headers = {
        'Authorization': auth_header
    }

    query = '{}/api/v2/object/{}({})/{}({})'.format(ilx_endpoint, object, id, navigation_property, related_id)
    response = r.get(query, headers=headers)

    return json.loads(response.text)


if __name__ == '__main__':
    pass



from setuptools import setup

with open('README.md', 'r') as fh:
    long_desc = fh.read()

setup(
    name='intelex',
    version='0.0.27',
    description='Intelex SDK',
    url='https://github.com/thomassampson/intelex_sdk',
    author='Thomas Sampson',
    author_email='sampsont91@gmail.com',
    py_modules=["intelex"],
    package_dir={'':'src'},
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    long_description=long_desc,
    long_description_content_type='text/markdown',
    install_requires = [
        'requests',
    ],
)